@@ -24,7 +24,7 @@ def includeme(config):
         config.set_authorization_policy(ACLAuthorizationPolicy())
         config.add_directive('register_authentication_policy',
                              register_auth_policy,
-                             action_wrap=True)
+                             action_wrap=False)
         config.commit()
 
 
@@ -45,31 +45,53 @@ class authentication_policy(object):
                                               self.context_cls_list)
 
 
-def register_auth_policy(config, policy, context_cls_list):
+def register_auth_policy(config, auth_policy, context_cls_list):
     ctx_policy = config.registry.getUtility(IAuthenticationPolicy)
-    ctx_policy.register_context(config, context_cls_list, policy)
+    ctx_policy.register_policy(config, auth_policy, context_cls_list)
 
 
 def get_authentication_policy(config):
+    # XXX should received registry as param.
     return config.registry.getUtility(IAuthenticationPolicy)
 
 
 class IContextBasedAuthenticationPolicy(IAuthenticationPolicy):
 
-    def register_context(self, context, auth_policy):
+    def register_policy(self, auth_policy, context):
         ""
 
 
 @implementer(IContextBasedAuthenticationPolicy)
 class ContextBasedAuthenticationPolicy(CallbackAuthenticationPolicy):
 
-    def register_context(self, config, context_cls_list, auth_policy):
+    intr_category = 'context based authentication policies'
+
+    def register_policy(self, config, auth_policy, context_cls_list):
         log.debug('registering auth_policy=%s for %s', auth_policy,
                   context_cls_list)
-        registry = config.registry
+
         if not isinstance(context_cls_list, collections.Iterable):
             context_cls_list = (context_cls_list, )
 
+        registry = config.registry
+        introspector = registry.introspector
+        policy_intr = introspector.get(self.intr_category, auth_policy)
+
+        if not policy_intr:
+            policy_intr = config.introspectable(
+                category_name=self.intr_category,
+                discriminator=auth_policy,
+                title=auth_policy,
+                type_name='authentication policy',
+            )
+            policy_intr['policy'] = auth_policy
+            policy_intr['contexts'] = []
+            config.action(None,
+                          introspectables=(policy_intr,),
+                          action_wrap=False)
+
+        policy_intr['contexts'].extend(context_cls_list)
+
         def factory(context):
             return auth_policy
 
@@ -88,6 +110,8 @@ class ContextBasedAuthenticationPolicy(CallbackAuthenticationPolicy):
         try:
             method = getattr(policy, method_name)
         except (KeyError, AttributeError):
+            log.debug('No authentication policy found: context=%s',
+                      request.context)
             return None
         return method(request, *args, **kwargs)
 
